function thetahat=anovahap(orggen,npops,N,loci,pops)

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

%ANOVA estimate for haploids
%follows Weir pp. 171-173

%inputs:
%orggen: contains allele frequency data
%npops: number of populations in sample
%pops: populations sampled
%N: number of individuals in each population
%loci: row vector containing indices of selected loci (can include duplicates)

%output:
%thetahat: column vector containing estimate for each locus 

numloci=size(loci,2);
loci=2.*(loci-1)+1;  %use first allele of those loci selected
orggen=orggen(loci,pops)/N;
%orggen=orggen(1:2:2*numloci,pops)

%Weir p. 171
msg=1/(npops*(N-1))*sum(N*orggen.*(1-orggen),2);
msp=1/(npops-1)*sum(N*(orggen-mean(orggen,2)*ones(1,npops)).^2,2);

%Weir p. 172
nc=1/(npops-1)*(npops*N-(npops*N^2)/(npops*N));
%Weir p.l 173
numerator=msp-msg;
denominator=msp+(nc-1)*msg;

for i=1:numloci
    if denominator(i)==0
        thetahat(i,1)=0.0;
    else
        thetahat(i,1)=numerator(i)/denominator(i);
    end
end

% function [thetahat]=anovahap(chlorfreqs,pops,loci,a,b)
% % anova estimate Fst for for haploids following Weir pp. 171-173 
% % inputs:
% % chlorfreqs: array containing proportions of genotypes/alleles
% % pops: vector containing indices of selected subpopulations (can include duplicates)
% % loci: vector containing indices of selected loci (can include duplicates)
% % a = number of populations, npops
% % b=sample size (N, same for each subpopulation; thus we ASSUME BALANCED DATA)
% % outputs:
% % thetahat: column vector containing point estimate of Fst (one row for each locus)
% % (NOTE FST IS CALCULATED FOR EACH LOCUS SEPARATELY)
% 
% numloci = length(loci);
% n = b*ones(a,1);%npops by one column vector of sample sizes
% for locus=2:numloci
%     n(:,:,locus)=n(:,:,1);  %extend our sample sizes out to multiple pages, one per locus
% end
% ntot=sum(n,1); % total sample size
% r=length(pops); % number of populations
% dfp=r-1;    % degrees of freedom between subpopulations
% 
% ptilde=chlorfreqs(pops,1,loci);    % frequency of allele A within subpopulations for each locus (locus is 3rd dimension here)
% pmeanest = sum(n.*ptilde,1)./ntot;   %overall pop mean of allele A freq for each locus
% 
% for i=1:numloci
%     locus = loci(1,i);  %look at the locus specified by the vector of loci
% 	if (pmeanest(1,1,locus) == 0) | (pmeanest(1,1,locus) == 1)    %sample of pops all have zero or one freq of A
%         %'pmean was zero for anova haploid'  %report zero
%         
%         thetahat(i,1)=0.0;
%     else
% 	
% 	pmean = repmat(pmeanest, r, 1);    %extend mean (ptilde A.) to all pops
% 	
% 	ssp = sum(n.*(ptilde-pmean).^2,1);
% 	msp=sum(n.*(ptilde-pmean).^2,1)/dfp; %same as Searle's msa
% 	
% 	dfg=ntot-r; %degrees of freedom between individuals within populations
% 	ssg = sum(n.*ptilde.*(1-ptilde),1);
% 	msg=sum(n.*ptilde.*(1-ptilde),1)./dfg; %same as Searle's mse
% 	
% 	nc=(ntot - (sum(n.^2,1)./ntot))/(r-1);
% 	
% 	t1almost=msp-msg;
% 	t1(:,1,1)=t1almost(1,1,:);  %take pages and stick them into rows
% 	t2almost=msp+(nc-1).*msg;
% 	t2(:,1,1)=t2almost(1,1,:);  %take pages and stick them into rows 
% 	
% 	for locus=1:numloci %error report if t2 is zero and therefore div zero for theta
%         if t2(locus,1,1) == 0
%             't2 is zero'    %report error
%             t1
%             t2
%             pmean
%             pops    %give sample of pops used
%             chlorfreqs
%         end
% 	end
% 
%     thetahat(i,1)=(t1./t2);
%     
%     end
% end


%test other method of computation
% nbar = sum(n,1)./r;
% s2A = ssp.*(1/((r-1).*nbar));
% 
% pmeanfactor = pmean(1,:,:).*(1-pmean(1,:,:));
% 
% altt1 = s2A-( (1./(nbar-1)).*(pmeanfactor - (((r-1)/r).*s2A)));
% 
% factor = ((r-1).*(nbar-nc))./(nbar-1);
% subt2 = (1 + factor).*(s2A./r);
% altt2 = ((nc-1)./(nbar-1)).*pmeanfactor + subt2;
% 
% alttheta = altt1./altt2;