function [bootlowbds, bootupbds]=bootconfint(thetareps, alpha, numreps)

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

% makes bootstrap confidence interval
% inputs:
% fname = name of estimator function (character string; put it in quotes)
% thetareps = vector or array of point estimates from bootstrap procedure
% dimensions: (locus, replicate estimate)
% (may be output of bootdip.m or boothap.m)
% alpha: request (1-alpha) confidence interval
% outputs:
% [bootlowbds, bootupbds] = lower and upper bounds on confidence interval
% dimensions: (locus) 
% note: recall that BOOTSTRAP OVER LOCI IS INAPPLICABLE FOR HAPLOIDS

%numreps=length(thetareps);
% length of confindence interval
intlength = min(round((1-alpha)*numreps),numreps);

% find best confidence interval
% note: for now, we find the SHORTEST INTERVAL in absolute terms.
numloci=size(thetareps);
numloci=numloci(1);

for locus=1:numloci % find CI for each locus separately
thetareps(locus,:)=sort(thetareps(locus,:));
bootlowbd=1;
bootupbd=intlength;
confwidth=thetareps(locus,bootupbd)-thetareps(locus,bootlowbd);
for i=2:numreps-intlength+1
    testwidth=thetareps(locus,i+intlength-1)-thetareps(locus,i);
    if testwidth<confwidth
        confwidth=testwidth;
        bootupbd=i+intlength-1;
        bootlowbd=i;
    end
end

% convert indices to confidence limits
bootlowbds(locus)=thetareps(locus,bootlowbd);
bootupbds(locus)=thetareps(locus,bootupbd);
end

% make column vectors
bootlowbds=bootlowbds.';
bootupbds=bootupbds.';  