function main
%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

%event loop to simulate population evolution, save results to file, and then estimate various
%subdivision measures and confidence intervals.

%population variables
nnloci=2;   %number of nuclear loci
norgloci=1; %number of organelle loci
totalpops=200;   %total number of populations in the finite model
npops=10;    %number of populations sampled
alpha = [1.0; 1.0];   %maternal transmission rate, norgloci by 1 column vector, one value for each organelle locus
N=20;       %number of individuals within each population
t=1.0;      %outcrossing rate
mp=0.12;     %pollen migration rate
ms=0.12;     %seed migration rate
gens=200;     %total number of generations to evolve
peek=5;     %generation interval to stop and collect data
bootreps = 1000;    %number of bootstrap replicates, must use at least one
sigalpha = 0.05;     %significance level

%Use below for initial freqs as long as all freqs are identical
pnuc = repmat(0.50, nnloci, totalpops); %the first value is the allele freq
porg = repmat(0.50, norgloci, totalpops);   %the first value is the allele freq

%Use when pops have different initial freqs.
%these init vectors have as many values as there are pops
% initpnuc = [0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5];   %initial nuclear frequencies
% initporg = [0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5];   %initial organelle frequencies
% 
% pnuc = [];
% for i=1:nnloci
%     pnuc = [pnuc; initpnuc];   %make nnloci by npops matrix of initial values
% end %for nloci loop
% 
% porg = [];
% for i=1:norgloci
%     porg = [porg; initporg];   %make norgloci by npops matrix of initial values
% end %for norgloci loop

%end section to declare variables

%begin section to simulate populations
needdata = 1;    %set to 1 to simulate new pops, 0 to use existing files

if needdata == 1
    [nucgen, orggen] = getgenotypes(pnuc, porg, nnloci, norgloci, totalpops, N, 1.0); %initialize nucgen and orggen
    %orggen
    [nucfreqs, chlorfreqs] = matt2judy(nucgen, orggen, nnloci, norgloci, N, totalpops); %initialize nucfreqs and chlorfreqs
    i = 1;  %specify this since it is saved in the file
	fname = sprintf('output%d',1);  %specify file name, each file has number starting with 1 as first
	save(fname);    %save our file in matlab binary
	
	for i=2:(gens/peek)+1
        %let populations evolve for peek generations
        [pnuc, porg, nucgen, orggen] = evolvepops(pnuc, porg, nnloci, norgloci, totalpops, alpha, N, t, mp, ms, peek);
        %'after evolvepops'
        %orggen

        %reworked version of matt2judy, changed chlorfreqs to spec
        [nucfreqs, chlorfreqs] = matt2judy(nucgen, orggen, nnloci, norgloci, N, totalpops);
        
        fname = sprintf('output%d',i);  %specify file name, each file has number starting with 1 as first
        save(fname);    %save our file in matlab binary
	
     end %for i=1:(gens/peek) loop
     
     disp('Data simulation completed....');
     
end %if needdata == true

gpnuc = zeros((gens/peek)+1, totalpops);    %allocate space for allele freqs to graph
gporg = zeros((gens/peek)+1, totalpops);

if needdata == 0;   %using existing data files
    npops = 8; %can vary the sample size here, must set here AND BELOW to same value
end

%select a random sample of pops to observe
temppops = randperm(totalpops); %get random permutation of total pops
pops = temppops(1,1:npops);     %select 1 to npops of temppops, this is our sample

%now let's estimate some stuff
for x = 1:(gens/peek)+1    %open our files in order
    
    disp('estimating subdivision for generation');
    scratch=x*peek;
    disp(scratch);
    
    fname = sprintf('output%d',x);  %specify file name, each file has number starting with 1 as first
    load(fname);    %load our matlab binary file

    %npops = 8; %can vary the sample size here, must set here since original value is stored in file
        
    %collect diploid and haploid allele freqs across gens
    gpnuc(x,:) = pnuc(1,:);
    gporg(x,:) = porg(1,:);
    %gporg2(x,:) = porg(2,:);    %get 2nd locus data
    
%do anova haploid
    %get point estimate with all data first
    loci = 1:norgloci;    %use all loci
    thetahap(:,x)=anovahap(orggen,npops,N,loci,pops);

    %then get POPULATION resampled estimates
	loci = 1:norgloci;  %estimate for all loci
	for y=1:bootreps
        rpops = resamplepops(npops,pops);
        thetahapreps(:,y)=anovahap(orggen,npops,N,loci,rpops);
    end
    [haplowbds(:,x), hapupbds(:,x)] = bootconfint(thetahapreps,sigalpha,bootreps);

%do anova diploid
    %get point estimate with all data first
    loci = 1:nnloci;    %use all loci, but could use a sample of loci
    [mlthetadip(1,x),thetadip(:,x)] = anovadip(nucfreqs,loci,pops,npops,N); 

    %then get POPULATION resampled estimates
	loci = 1:nnloci;  %estimate for all loci
	for y=1:bootreps
        rpops = resamplepops(npops,pops);
        [mlthetadipreps(1,y),thetadipreps(:,y)]=anovadip(nucfreqs,loci,rpops,npops,N);
    end
    [diplowbds(:,x), dipupbds(:,x)] = bootconfint(thetadipreps,sigalpha,bootreps);
    [mldiplowbds(:,x), mldipupbds(:,x)] = bootconfint(mlthetadipreps,sigalpha,bootreps);

%do Nei diploid
    %get point estimate with all data first
    loci = 1:nnloci;    %use all loci
    Gstdip(1,x)=neidip(nucgen,pops,N,nnloci,npops);

    %then get resampled estimates
	loci = 1:nnloci;  %estimate for all loci
	for y=1:bootreps
        rpops = resamplepops(npops,pops);
        rGstdip(1,y)=neidip(nucgen,rpops,N,nnloci,npops);	
    end
    [neidiplowbds(:,x), neidipupbds(:,x)] = bootconfint(rGstdip,sigalpha,bootreps);
    
%do Nei-ish haploid
    %get point estimate with all data first
    loci = 1:norgloci;
    %[Gsthap(:,x)] = ponshap(chlorfreqs,pops,loci,npops,N,norgloci);
    Gsthap(:,x)=ponshap(orggen,npops,N,loci,pops);

	%then get resampled estimates
	loci = 1:norgloci;  %estimate for all loci
	for y=1:bootreps
        rpops = resamplepops(npops,pops);
        rGsthap(:,y)=ponshap(orggen,npops,N,loci,rpops);
	end
    [neihaplowbds(:,x), neihapupbds(:,x)] = bootconfint(rGsthap,sigalpha,bootreps);
    
end %for i=1:(gens/peek)+1 loop

%now make some tables of output
% gpnuc   %allele freqs
% gporg
% thetahap    %hapolid theta
% haplowbds
% hapupbds
% thetadip    %diploid theta
% diplowbds
% dipupbds
% Gstdip     %diploid Gst
% neidiplowbds
% neidipupbds
% Gsthap      %haploid Gst
% neihaplowbds
% neihapupbds

% now make some figures
figure(1);%theta for organelle and diploid
hold on;
plot(peek*(0:gens/peek),thetahap(1,:),'k.-');   %plot haploid point estimate
plot(peek*(0:gens/peek),haplowbds(1,:),'k--');  %plot haploid lower bound
plot(peek*(0:gens/peek),thetadip(1,:),'k*-');    %plot diploid point estimate
plot(peek*(0:gens/peek),diplowbds(1,:),'k:');   %plot diploid lower bound
plot(peek*(0:gens/peek),dipupbds(1,:),'k:');    %plot diploid upper bound
plot(peek*(0:gens/peek),hapupbds(1,:),'k--');   %plot haploid upper bound
legend('Haploid \Theta','Haploid 95% CI','Diploid \Theta','Diploid 95% CI');

axis([0 gens+1 -0.1 1.1]);
title('\Theta (theta) for one diploid locus and one haploid locus');
xlabel('Generations');
ylabel('Theta (\Theta)');
box on;
hold off;

% figure(2);  %Gst for organelle and diploid
% hold on;
% plot(peek*(0:gens/peek),Gsthap(1,:),'k.-');    %plot nei haploid point estimate
% plot(peek*(0:gens/peek),neihaplowbds(1,:),'k--');   %plot hap lower bound
% plot(peek*(0:gens/peek),Gstdip(1,:),'k*-');    %plot nei diploid point estimate
% plot(peek*(0:gens/peek),neidiplowbds(1,:),'k:');   %plot dip lower bound
% plot(peek*(0:gens/peek),neidipupbds(1,:),'k:');    %plot dip upper bound
% plot(peek*(0:gens/peek),neihapupbds(1,:),'k--');    %plot hap upper bound
% legend('Haploid Gst','Haploid Gst 95% CI','Diploid Gst','Diploid Gst 95% CI');
% axis([0 gens+1 -0.1 1.1]);
% title('Nei''s diploid Gst and Pons/Petit haploid Gst from one locus');
% xlabel('Generations');
% ylabel('Gst');
% box on;
% hold off;

if nnloci>1 %do for more than one nuclear locus only
% 	figure(3);%multilocus theta and Gst (diploid)
% 	hold on;    
% 	plot(peek*(0:gens/peek),mlthetadip(1,:),'k*-');    %plot ml diploid point estimate
% 	plot(peek*(0:gens/peek),diplowbds(1,:),'k:');   %plot ml diploid lower bound
%     plot(peek*(0:gens/peek),Gstdip(1,:),'k.-');    %plot nei diploid point estimate
%     plot(peek*(0:gens/peek),neidiplowbds(1,:),'k--');   %plot Gst dip lower bound
%     plot(peek*(0:gens/peek),neidipupbds(1,:),'k--');    %plot Gst dip upper bound
% 	plot(peek*(0:gens/peek),dipupbds(1,:),'k:');    %plot ml diploid upper bound
%     legend('ML \Theta','ML \Theta 95% CI','Gst','Gst 95% CI');
% 	
% 	axis([0 gens+1 -0.1 1.1]);
% 	title('multilocus \Theta (theta) and Gst for all nuclear loci');
% 	xlabel('Generations');
% 	ylabel('ML Theta (\Theta) and Gst');
% 	box on;
% 	hold off;
    
    figure(8);%nuclear multilocus theta and haploid theta
	hold on;    
   	plot(peek*(0:gens/peek),mlthetadip(1,:),'k*-');    %plot ml diploid point estimate
	plot(peek*(0:gens/peek),mldiplowbds(1,:),'k:');   %plot ml diploid lower bound
    plot(peek*(0:gens/peek),thetahap(1,:),'k.-');   %plot haploid point estimate
    plot(peek*(0:gens/peek),haplowbds(1,:),'k--');  %plot haploid lower bound
    plot(peek*(0:gens/peek),hapupbds(1,:),'k--');   %plot haploid upper bound
    plot(peek*(0:gens/peek),mldipupbds(1,:),'k:');    %plot ml diploid upper bound
    legend('ML \Theta','ML \Theta 95% CI','Haploid \Theta','Haploid 95% CI');
	
	axis([0 gens+1 -0.1 1.1]);
	title('multilocus \Theta (theta) for all nuclear loci and organelle \Theta (theta)');
	xlabel('Generations');
	ylabel('ML and haploid Theta (\Theta)');
	box on;
	hold off;

end

%plot allele frquencies of locus 1 for organelle and diploid using all pops (totalpops)
% figure(4);
% hold on;
% plot(peek*(0:gens/peek),gpnuc(:,1),'rd-');   %plot nuclear allele freqs
% plot(peek*(0:gens/peek),gporg(:,1),'kx-');   %plot organelle allele freqs
% for x=2:totalpops
%     plot(peek*(0:gens/peek),gpnuc(:,x),'rd-');   %plot nuclear allele freqs
%     plot(peek*(0:gens/peek),gporg(:,x),'kx-');   %plot organelle allele freqs
% end
% legend('Nuclear locus','Organelle locus');
% 
% axis([0 gens+1 -0.05 1.05]);
% title('allele freqs for one locus all pops: nuclear (diamonds) and organelle (cross)');
% xlabel('Generations');
% ylabel('Frequency of A allele (p)');
% box on;
% hold off;

%plot allele frquencies of locus 1 for organelle and diploid using sampled pops only (npops)
figure(5);
hold on;
plot(peek*(0:gens/peek),gpnuc(:,pops(1,1)),'rd-');   %plot nuclear allele freqs
plot(peek*(0:gens/peek),gporg(:,pops(1,1)),'kx-');   %plot organelle allele freqs
for x=2:npops
    plot(peek*(0:gens/peek),gpnuc(:,pops(1,x)),'rd-');   %plot nuclear allele freqs
    plot(peek*(0:gens/peek),gporg(:,pops(1,x)),'kx-');   %plot organelle allele freqs
end
legend('Nuclear locus','Organelle locus');

axis([0 gens+1 -0.05 1.05]);
title('allele freqs for one locus from npops sampled for estimates');
xlabel('Generations');
ylabel('Frequency of A allele (p)');
box on;
hold off;

%plot histrogram of resampled Fst estimates used for CI
% figure(6);
% bins = -0.2:0.05:1.2;
% hist(thetahapreps(1,:), bins);
% title('distribution of resampled Fst estimates at last sample: anova hap'); 

%plot allele frquencies of locus 1&2 for organelle
% figure(7);
% hold on;
% plot(peek*(0:gens/peek),gporg(:,1),'kd-');   %plot nuclear allele freqs
% plot(peek*(0:gens/peek),gporg2(:,1),'kx-');   %plot organelle allele freqs
% for x=2:npops
%     plot(peek*(0:gens/peek),gporg(:,x),'kd-');   %plot nuclear allele freqs
%     plot(peek*(0:gens/peek),gporg2(:,x),'kx-');   %plot organelle allele freqs
% end
% legend('org locus 1','org locus 2');
% 
% axis([0 gens+1 -0.05 1.05]);
% title('allele freqs for two organelle loci');
% xlabel('Generations');
% ylabel('Frequency of A allele (p)');
% box on;
% hold off;

function [rpops] = resamplepops(npops,pops)
%resample populations with replacement for bootstrap
%test to see if resampled pops are all the same population,
%if so resample again

%inputs:
%npops = the number of populations sampled
%pops = the row vector of indices of sampled populations

    dup=1;
    while dup==1
		index = unidrnd(npops,1,npops); % get random set of npops indices
		rpops = pops(1,index); % assign pops based on indices, or sample with replacement from pops
        mx=max(rpops);   %get first index of maximum value
        i=find(rpops==mx);   %get all other indices that are equal to maximum value
        numdups = length(i);   %get size of array of indices that are equal to max
        if numdups==npops
            dup=1;  %all pops are the same
        else
            dup=0;  %all pops are not the same
        end
    end
    

function [nucgen, orggen] = getgenotypes(pnuc, porg, nnloci, norgloci, numpops, N, t);    
%make genotypes from allele frequencies, use only to make initial values of nucgen and orggen
%full version in evolvepops.m
%nuclear genotype matrix has three rows for each genotytpe (AA, Aa, aa) per locus
%organelle genotype matrix has one row for each genotytpe (A) per locus
%each column is one population in both cases
%mating is random for t percent of genotypes and selfing for 1-t percent
    
    %make nuclear genotypes
    nucgen = zeros(3*nnloci,numpops);
    i = 0;  %index for rows in pnuc
    for k=1:3:3*nnloci
        i = i+1;
        for j=1:numpops
            het = round(t*(2*pnuc(i,j)*(1-pnuc(i,j)))*N);    %number of Aa that mate randomly
            selfedhet = round((1-t)*(2*pnuc(i,j)*(1-pnuc(i,j)))*N);  %number of Aa that self
            nucgen(k,j) = round(pnuc(i,j)^2*N)+round(0.25*selfedhet);    %number of AA
            nucgen(k+1,j) = het+round(0.50*selfedhet);    %number of Aa
            nucgen(k+2,j) = round((1-pnuc(i,j))^2*N)+round(0.25*selfedhet);    %number of aa
        end %end numpops loop
    end %end loci loop
    
    %make organelle genotypes
    i = 0;  %index for rows in porg
    orggen = zeros(2*norgloci,numpops);
    for k=1:2:2*norgloci
        i = i+1;
        for j=1:numpops
            orggen(k,j) = round(porg(i,j)*N);    %number of A haplotypes
            orggen(k+1,j) = N - orggen(k,j);    %number of a haplotypes
        end %end numpops loop
    end %end loci loop
