function [nucfreqs, chlorfreqs]=matt2judy(nucgen, orggen, nnloci, norgloci, N, npops)

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

% May 7, 2001
% converts Matt's output to Judy's input
%
% inputs:
% nucgen: contains nuclear genotype counts
% dimensions: (nnloci*3,npops)
% orggen: contain organelle genotype counts
% dimensions: (norgloci*2, npops)
%
% outputs:
% nucfreqs: contains nuclear (diploid) genotype frequencies
% dimensions (population, locus, genotype(pAA pAa paa)
% chlorfreqs: contains chloroplast (organelle, haploid) genotype frequencies
% dimensions (population, genotype(pA pa), locus)

% nuclear (diploid) conversion
for i=1:3
    nucfreqs(:,:,i)=nucgen(i:3:i+3*(nnloci-1),:).';
end
nucfreqs=nucfreqs/N;

%error check nuclear frequencies (freqs at one locus should sum to one)
% for i=1:npops   %for all pops   
%     for j=1:nnloci  %for all loci
%         total = 0.0;
%         for k=1:3   %for all genotypes
%             total=nucfreqs(i,j,k)+total;
%         end
%         if (total > 1.02) | (total < 0.98) %range assumes ample rounding error
%             'diploid genotype freq out of range'  %report error
%             i
%             j
%             k
%             total
%         end
%     end
% end

% chloroplast (haploid, organelle) conversion
torggen = orggen.';  %tranpose orggen, now pops in rows and loci in columns
for j=1:1:norgloci
    chlorfreqs(:,1,j) = torggen(:,2*j-1);   %get count of A
    chlorfreqs(:,2,j) = torggen(:,2*j);   %get count of a
end
chlorfreqs=chlorfreqs/N;

%error check organelle frequencies (freqs at one locus should sum to one)
%(population, genotype(pA pa), locus)
for i=1:npops   %for all pops   
    for j=1:norgloci  %for all loci
        total = 0.0;
        for k=1:2   %for all genotypes
            total=chlorfreqs(i,k,j)+total;
        end
        if (total > 1.05) or (total < 0.95) %range assumes ample error
            'organelle genotype freq out of range'  %report error
            i
            j
            k
            total
        end
    end
end