function [gsthat]=neidip(nucgen,pops,N,nnloci,npops)

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

%,h0hat,hshat,hthat,hshatavg,hthatavg]
% Gst estimate of Fst for for diploids
% follows Nei & Kumar, 2000 p. 241, equations (12.25)-(12.28)
% We assume all pops are of equal size.

nucgen=nucgen(:,pops)/N; %use only the populations in the sample, and convert counts to percentages
nucalleles=nucgen(1:3:3*nnloci-2,:)+.5*nucgen(2:3:3*nnloci-1,:); %convert nucgen to allele frequencies
AAhomozygs=nucgen(1:3:3*nnloci-2,:); %percentage of AA homozygotes
aahomozygs=nucgen(3:3:3*nnloci,:); %percentage of aa homozygotes

%h0hat=1-sum(1/npops.*(AAhomozygs+aahomozygs),2) %(12.25)
h0hat=1-mean(AAhomozygs+aahomozygs,2); %(12.25)
hshat=N/(N-1)*(1-mean(nucalleles.^2+(1-nucalleles).^2,2)-h0hat/(2*N)); %(12.26)
hthat=1-(mean(nucalleles,2).^2+mean(1-nucalleles,2).^2)+hshat/(N*npops)-h0hat/(2*N*npops); %(12.27)

hshatavg=mean(hshat);
hthatavg=mean(hthat);

if hthatavg==0
    gsthat=0;
else
    gsthat=1-hshatavg/hthatavg; %(12.28c)
    
    %the code below will adjust Gst for small number of populations
    %the adjustment makes little difference if npops>=5
%   dstprimehat=(npops/(npops-1))*(hthat-hshat);
% 	htprimehat=dstprimehat+hshat;
% 	dstprimehatavg=mean(dstprimehat);
% 	htprimehatavg=mean(htprimehat);
% 	gstprimehat=dstprimehatavg/htprimehatavg;
    
end