function gsthat=ponshap(orggen,npops,N,loci,pops)

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

%Gst estimate for haploids
%follows Pons/Petit (1995)

%inputs:
%orggen: contains allele frequency data
%npops: number of populations in sample
%pops: populations sampled
%N: number of individuals in each population
%loci: row vector containing indices of selected loci (can include duplicates)

%output:
%gsthat: column vector containing estimate for each locus 

numloci=size(loci,2);
loci=2.*(loci-1)+1;  %use first allele of those loci selected
orggen=orggen(loci,pops)/N;
%orggen=orggen(:,pops)/N %use only the populations in the sample, and convert counts to percentages

hshat=1/npops*(N/(N-1))*sum(1-(orggen.^2+(1-orggen).^2),2); % equation (6) of Pons/Petit

hthat=1-(mean(orggen,2).^2 +mean(1-orggen,2).^2)+...
    1/(npops*(npops-1))*(sum((orggen-mean(orggen,2)*ones(1,npops)).^2,2)+...
        +sum((1-orggen-mean(1-orggen,2)*ones(1,npops)).^2,2)); % equation (9) of Pons/Petit        

for i=1:numloci
	if hthat(i)==0 %there is no genetic variation
        gsthat(i,1)=0.0; %assign value directly to avoid div zero
	else
        gsthat(i,1)=1-hshat(i,1)/hthat(i,1); % equation (10) of Pons/Petit
	end
end

% function [thetahat]=ponshap(chlorfreqs,pops,loci,a,b,norgloci)
% % estimate Gst for for haploids following Pons/Petit (1995)
% 
% % inputs:
% % chlorfreqs: array containing proportions of genotypes/alleles 
% % pops: vector containing indices of selected subpopulations (can include duplicates)
% % loci: 
% % a = number of populations, or npops
% % b=sample size within pops (N, same for each subpopulation; thus we ASSUME BALANCED DATA)
% 
% % outputs:
% % thetahat: column vector containing point estimate of Fst (one row for each locus)
% % (NOTE FST IS CALCULATED FOR EACH LOCUS SEPARATELY)
% % (ALSO NOTE CODE CURRENTLY WORKS ONLY FOR TWO ALLELES, NOT MORE)
% 
% numloci=norgloci;
% 
% nk=b*ones(a,1); % sample sizes
% for locus=2:numloci
%     nk(:,:,locus)=nk(:,:,1);
% end
% nktot=sum(nk,1); % total sample size
% numpops=length(pops); % number of populations
% 
% xki=chlorfreqs(pops,1,:); % frequency of allele A within subpopulations for each locus (locus is 3rd dimension here)
% xdoti=sum(nk.*xki,1)./nktot; %overall percentage of allele A for each locus
% for pop=2:numpops   
%     xdoti(pop,:,:)=xdoti(1,:,:);
% end
% 
% % h_s^\hat (eqn. 6 of Pons/Petit)
% sumxkisq=xki.^2+(1-xki).^2; % WORKS FOR TWO ALLELES ONLY
% hshat=1/numpops*sum(nk./(nk-1).*(1-sumxkisq),1);
% 
% % h_t^\hat (eqn.9 of Pons/Petit)
% hthat=1-(xdoti.^2+(1-xdoti).^2);hthat=hthat(1,:,:);
% hthat=hthat+1/numpops/(numpops-1)*sum((xki-xdoti).^2+(1-xki-xdoti).^2,1); % WORKS FOR TWO ALLELES ONLY
% 
% % G_{ST}^\hat (eqn. 10 of Pons/Petit)
% thetahatalmost=1-hshat./hthat; 
% thetahat(:,1,1)=thetahatalmost(1,1,:);